
using System;
using System.Windows.Forms;
using System.Text;
using System.Text.RegularExpressions;
using System.Drawing;
using System.IO;
using NppScripts;


public class MyForm:Form
{
	public static string WorkingFolder  =  @"";
	
	public static Color RED = Color.FromArgb(255,200,10,10);
	
	public const string THIS_SCRIPT = "DependencesInScripts";
	
	public TreeView TV1;
	
	public static string GetScriptName(string path)
	{
		int i = path.LastIndexOf(@"\");
		if (i<0)
		{return path;} else {
			string s = path.Substring(i+1,path.Length-i-1); 
			return s;
		}	
	}
	
	
	public MyForm()
	{
		this.Width=700;
		this.Height=500;
		this.Text="Dependences";
		TV1 = new TreeView();
		TV1.Width=this.Width-5;
		TV1.Height=this.Height-5;
		this.Controls.Add(TV1);
		this.Load += makeTV;
	}
	
	
	private void onNodeSelection(Object sender, TreeViewEventArgs e)
	{
		if (e.Action==TreeViewAction.ByMouse)
		{
			string path=(sender as TreeView).SelectedNode.Name;
			if (File.Exists(path))
			{
		       Win32.SendMessage(Npp.NppHandle, NppMsg.NPPM_DOOPEN, 0, path);
			}	
		}
	}
	
	
	public static string SQS_FullPath(string path)
	{
		if (path.Length<5)
		{
			return "";
		}
		
		string pth = GetScriptName(path);
		string[] files;
		
		files = Directory.GetFiles(WorkingFolder, @"*" + pth, SearchOption.AllDirectories);
		
		foreach (string f in files)
		{
			if (f.EndsWith(path,StringComparison.InvariantCultureIgnoreCase))
			{
				return f;
			}	
		}
		return "";	
	}
	
	
	 private static string NoComments(string s)
	 {
		  string s2;
		  Regex rex;
		  rex=new Regex(@"\/{2,}[\w \t]+");		  
		  s2=rex.Replace(s,"");
		  
		  rex=new Regex(@"\/\*(.|\n)+?\*\/");
		  s2=rex.Replace(s2,"");
		  
		  rex=new Regex(@"comment\s*?[\{](.|\n)+?\}\;",RegexOptions.IgnoreCase);
		  s2=rex.Replace(s2,"");
		  
		  rex=new Regex(@"^\;.+$", RegexOptions.Multiline);
		  s2=rex.Replace(s2,"");
		  return(s2);
	 }

	 
 	public void AddDepended(TreeNode node)
	{
		if (node!=null)
		{
			string file = node.Name;
			if (File.Exists(file))
			{
				string text = File.ReadAllText(file);
				text = NoComments(text);
				
				//string s = @"(\{\\|\""\\).+?\.[sS][qQ][sfSF]((\}|\""))";
				string s = @"((\{|\""))[\\ \w]+?\.[sS][qQ][sfSF]((\}|\""))";
				
				Regex rex = new Regex(s);
				foreach (Match M in rex.Matches(text))
				{
					string path = M.Value;
					string key = path.Substring(1, path.Length-2);
					string txt = SQS_FullPath(key);
					
					bool red = false;
					if (txt=="")
					{
						txt = key + " (not found)";
						red=true;
					}
					
					if (!(node.Nodes.ContainsKey(txt)))
					{
						if (node.Name==txt)
						{
							node.Nodes.Add(txt+" (recursion)", txt+" (recursion)");	
						}	
						else
						{
							node.Nodes.Add(txt, txt);
				
							if (red)
							{
								node.Nodes[txt].ForeColor=RED;
							} else {
								AddDepended(node.Nodes[txt]);
							}	
							
						}
								
					}		
					
				}
				
			}		
				
		}	
	}

	
	public void makeTV(object sender, System.EventArgs e)
    {
		if (CheckWorkingFolder())
		{
			TV1.BeginUpdate();
			string path = Npp.GetCurrentDocument();
			TV1.Nodes.Add(path, path);
			AddDepended(TV1.Nodes[path]);
			TV1.EndUpdate();
			TV1.ExpandAll();
			TV1.AfterSelect += onNodeSelection;
		}	
    } 
	
	
	public static string TextOfNPPScript(string nm)
	{
		string MyDocs =  System.Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments); 
	    string dir = MyDocs+ @"\NppScripts";
		string name = nm + ".cs";
		string[] files = Directory.GetFiles(dir, @"*" + name);
		
		string file = "";
		if (files.Length>0)
		{
			file = File.ReadAllText(files[0]);
		}
		return file;	
	}
	
	
	public static string ReplacePublicString(string txt, string v, string val)
	{
		string s = @"public static string " + v;
		string Q = @"""";
		Regex rex = new Regex(s + @".+\;");
		string val2 = s + @"  =  @" + Q + val + Q + @";";
		string txt2 = rex.Replace(txt,val2);
		return(txt2);
	}
	
	
	public static void WriteNPPScript(string nm, string text)
	{
		string MyDocs =  System.Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments); 
	    string dir = MyDocs+ @"\NppScripts";
		string name = nm + ".cs";
		string[] files = Directory.GetFiles(dir, @"*" + name);
		if (files.Length>0)
		{
			string file = files[0];
			File.WriteAllText(file, text);
		}	
	}
	
	
	public static string GetWorkingFolder()
	{
		FolderBrowserDialog fbd = new FolderBrowserDialog(); 
		fbd.RootFolder = Environment.SpecialFolder.MyComputer; 
		fbd.Description = "Select working folder"; 
		if (fbd.ShowDialog() == DialogResult.OK) 
		{
			return fbd.SelectedPath;
		}
		else 
		{
			return "";
		}	
	}
	
	
	public static bool CheckWorkingFolder()
	{
		if (!(Directory.Exists(WorkingFolder)))
		{
			string wf = GetWorkingFolder();
			WorkingFolder = wf;
			
			string txt = TextOfNPPScript (THIS_SCRIPT);

			txt = ReplacePublicString
			(
				txt,
				"WorkingFolder", 
				wf
			);
			WriteNPPScript(THIS_SCRIPT, txt);
		}
		
		if (Npp.GetCurrentDocument().StartsWith(WorkingFolder))
		{return true;}
		else
		{
			MessageBox.Show(
			"Place all scripts into your working folder" + 
			Environment.NewLine+WorkingFolder
			);
			return false;
		}
	} 
}


public class Script : NppScript
{
    public override void Run()
    {
		Form MainForm = new MyForm();
        Application.Run(MainForm);
    } 
}
